package org.apache.commons.lang3;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.EnumSet;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.Validate;

public class EnumUtils {
  private static final String NULL_ELEMENTS_NOT_PERMITTED = "null elements not permitted";
  
  private static final String CANNOT_STORE_S_S_VALUES_IN_S_BITS = "Cannot store %s %s values in %s bits";
  
  private static final String S_DOES_NOT_SEEM_TO_BE_AN_ENUM_TYPE = "%s does not seem to be an Enum type";
  
  private static final String ENUM_CLASS_MUST_BE_DEFINED = "EnumClass must be defined.";
  
  private static <E extends Enum<E>> Class<E> asEnum(Class<E> enumClass) {
    Validate.notNull(enumClass, "EnumClass must be defined.", new Object[0]);
    Validate.isTrue(enumClass.isEnum(), "%s does not seem to be an Enum type", new Object[] { enumClass });
    return enumClass;
  }
  
  private static <E extends Enum<E>> Class<E> checkBitVectorable(Class<E> enumClass) {
    Enum[] arrayOfEnum = asEnum(enumClass).getEnumConstants();
    Validate.isTrue((arrayOfEnum.length <= 64), "Cannot store %s %s values in %s bits", new Object[] { Integer.valueOf(arrayOfEnum.length), enumClass.getSimpleName(), Integer.valueOf(64) });
    return enumClass;
  }
  
  @SafeVarargs
  public static <E extends Enum<E>> long generateBitVector(Class<E> enumClass, E... values) {
    Validate.noNullElements((Object[])values);
    return generateBitVector(enumClass, Arrays.asList(values));
  }
  
  public static <E extends Enum<E>> long generateBitVector(Class<E> enumClass, Iterable<? extends E> values) {
    checkBitVectorable(enumClass);
    Validate.notNull(values);
    long total = 0L;
    for (Enum enum_ : values) {
      Validate.notNull(enum_, "null elements not permitted", new Object[0]);
      total |= 1L << enum_.ordinal();
    } 
    return total;
  }
  
  @SafeVarargs
  public static <E extends Enum<E>> long[] generateBitVectors(Class<E> enumClass, E... values) {
    asEnum(enumClass);
    Validate.noNullElements((Object[])values);
    EnumSet<E> condensed = EnumSet.noneOf(enumClass);
    Collections.addAll(condensed, values);
    long[] result = new long[(((Enum[])enumClass.getEnumConstants()).length - 1) / 64 + 1];
    for (Enum enum_ : condensed)
      result[enum_.ordinal() / 64] = result[enum_.ordinal() / 64] | 1L << enum_.ordinal() % 64; 
    ArrayUtils.reverse(result);
    return result;
  }
  
  public static <E extends Enum<E>> long[] generateBitVectors(Class<E> enumClass, Iterable<? extends E> values) {
    asEnum(enumClass);
    Validate.notNull(values);
    EnumSet<E> condensed = EnumSet.noneOf(enumClass);
    for (Enum enum_ : values) {
      Validate.notNull(enum_, "null elements not permitted", new Object[0]);
      condensed.add((E)enum_);
    } 
    long[] result = new long[(((Enum[])enumClass.getEnumConstants()).length - 1) / 64 + 1];
    for (Enum enum_ : condensed)
      result[enum_.ordinal() / 64] = result[enum_.ordinal() / 64] | 1L << enum_.ordinal() % 64; 
    ArrayUtils.reverse(result);
    return result;
  }
  
  public static <E extends Enum<E>> E getEnum(Class<E> enumClass, String enumName) {
    return getEnum(enumClass, enumName, null);
  }
  
  public static <E extends Enum<E>> E getEnum(Class<E> enumClass, String enumName, E defaultEnum) {
    if (enumName == null)
      return defaultEnum; 
    try {
      return Enum.valueOf(enumClass, enumName);
    } catch (IllegalArgumentException ex) {
      return defaultEnum;
    } 
  }
  
  public static <E extends Enum<E>> E getEnumIgnoreCase(Class<E> enumClass, String enumName) {
    return getEnumIgnoreCase(enumClass, enumName, null);
  }
  
  public static <E extends Enum<E>> E getEnumIgnoreCase(Class<E> enumClass, String enumName, E defaultEnum) {
    if (enumName == null || !enumClass.isEnum())
      return defaultEnum; 
    for (Enum enum_ : (Enum[])enumClass.getEnumConstants()) {
      if (enum_.name().equalsIgnoreCase(enumName))
        return (E)enum_; 
    } 
    return defaultEnum;
  }
  
  public static <E extends Enum<E>> List<E> getEnumList(Class<E> enumClass) {
    return new ArrayList<>(Arrays.asList(enumClass.getEnumConstants()));
  }
  
  public static <E extends Enum<E>> Map<String, E> getEnumMap(Class<E> enumClass) {
    Map<String, E> map = new LinkedHashMap<>();
    for (Enum enum_ : (Enum[])enumClass.getEnumConstants())
      map.put(enum_.name(), (E)enum_); 
    return map;
  }
  
  public static <E extends Enum<E>> boolean isValidEnum(Class<E> enumClass, String enumName) {
    return (getEnum(enumClass, enumName) != null);
  }
  
  public static <E extends Enum<E>> boolean isValidEnumIgnoreCase(Class<E> enumClass, String enumName) {
    return (getEnumIgnoreCase(enumClass, enumName) != null);
  }
  
  public static <E extends Enum<E>> EnumSet<E> processBitVector(Class<E> enumClass, long value) {
    checkBitVectorable(enumClass).getEnumConstants();
    return processBitVectors(enumClass, new long[] { value });
  }
  
  public static <E extends Enum<E>> EnumSet<E> processBitVectors(Class<E> enumClass, long... values) {
    EnumSet<E> results = EnumSet.noneOf(asEnum(enumClass));
    long[] lvalues = ArrayUtils.clone((long[])Validate.notNull(values));
    ArrayUtils.reverse(lvalues);
    for (Enum enum_ : (Enum[])enumClass.getEnumConstants()) {
      int block = enum_.ordinal() / 64;
      if (block < lvalues.length && (lvalues[block] & 1L << enum_.ordinal() % 64) != 0L)
        results.add((E)enum_); 
    } 
    return results;
  }
}
